###-------------- Defines necessary funtions  ----------------------------------
# The following functions runs on R version 2.9.1
#
# Author(s): G. Nuez-Antonio and E. Gutirrez-Pea < eduardo@sigma.iimas.unam.mx >
# Maintainer: G. Nuez-Antonio < gab.nuneza@gmail.com >
# Date: July, 2009.
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------

CircReg<-function(mod1,mod2=mod1,data,tm,t.lag=1,burn=2000,flag.lk=FALSE){

# This function performs a Gibbs sampling to sample from posterior
# densities concerning the circular regression model based on a projected normal
# distribution. See, "A Bayesian Regression Model for
# Circular Data based on the projected Normal Distribution" of Nuez-Antonio G.,
# Gutirrez-Pea E., and Escarela G.

# Arguments:
#           mod1, mod2: Specification of component-I and component-II, respectively.
#           data: A data frame.
#           tm: Total sample size.
#           t.lag: The sample is taken every t.lag-th iteration.
#           burn: Number of iterations as burn-in period for Gibbs algorithm.
#           flag.lk: logical; if TRUE, the predictive measure lk is computed.

# Value:
#       CircReg returns a list with the following components:
#       BI: A matrix containing a sample of coefficients of the models component-I.
#       BII: A matrix containing a sample of coefficients of the models component-II.
#       Lk: The value of the predictive measure lk.


# Details:
# mod1 and mod 2 are objects of class "formula". See, R (2009), for example.
# If mode 2 is not specified, it assumes the same structure as mod1.
# If t.lag or burn or flag.lk is not specified, assume the default values
# of 1, 2000 and FALSE, respectively.
# If flag.lk is not TRUE, the return final value is lk=0.


# Author(s): G. Nuez-Antonio and E. Gutirrez-Pea < eduardo@sigma.iimas.unam.mx >
# Maintainer: G. Nuez-Antonio < gab.nuneza@gmail.com >
# Date: July, 2009.
#
#-------------------------------------------------------------------------------


library(MASS)

# Data Matrix

datose<-cbind(cos(data$theta),sin(data$theta))
n<-length(data$theta)

# Design Matrices

X.1.aux<-model.matrix(mod1,data)
p.1<-length(X.1.aux[1,])
X.1<-matrix(X.1.aux,ncol=p.1)

X.2.aux<-model.matrix(mod2,data)
p.2<-length(X.2.aux[1,])
X.2<-matrix(X.2.aux,ncol=p.2)

# Prior specification for "matrix" B.

m1<-rep(0,p.1)
L1<-0.0001*diag(p.1)
m2<-rep(0,p.2)
L2<-0.0001*diag(p.2)

# Posterior specification for "matrix" B.

XtX.1<-crossprod(X.1)
XtX.2<-crossprod(X.2)
Lstar1<-L1+XtX.1
Lstar2<-L2+XtX.2
Sigma1<-chol2inv(chol(Lstar1))
Sigma2<-chol2inv(chol(Lstar2))
L1m1<-crossprod(L1,m1)
L2m2<-crossprod(L2,m2)

###----------------------  GIBBS SAMPLING   -----------------------------------
#cat("Starting Gibbs Sampler ....", fill = TRUE)

### Initial values

r<-rep(1,n)

### Number of iterations

kk<-tm*t.lag
print(paste(" Total iterations =", kk, " ..."))

### Matrices for getting final sample.

B1<-matrix(0,tm,p.1)
B2<-matrix(0,tm,p.2)

#------------------ Burn-in period ----------------------

for(k in 1:burn)
{

Y<-r*datose

# Sampling of vectors beta1 and beta2.

XtY1<-crossprod(X.1,Y[,1])
XtY2<-crossprod(X.2,Y[,2])
mstar1<-c( crossprod(Sigma1,((L1m1)+XtY1) ) )
mstar2<-c( crossprod(Sigma2,((L2m2)+XtY2) ) )
beta1<-mvrnorm(1,mstar1,Sigma1)
beta2<-mvrnorm(1,mstar2,Sigma2)


# Sampling of vector r

for(j in 1:n)
  {
    t.aux<-data$theta[j]
    mu.b1<-c(crossprod(beta1,X.1[j,]))
    mu.b2<-c(crossprod(beta2,X.2[j,]))
    b<-Dbd(t.aux,mu.b1,mu.b2)
    r[j]<-b+(  pnorm(b)/( dnorm(b)+b*pnorm(b) )   )
  }

}

### ---------------------------- Iterations ------------------------------------

for(k in 1:kk)
{

flag1<-(k/1000)-trunc(k/1000)
if(flag1==0){print(k)}


# Sampling of vectors beta1 and beta2.
XtY1<-crossprod(X.1,Y[,1])
XtY2<-crossprod(X.2,Y[,2])
mstar1<-c( crossprod(Sigma1,((L1m1)+XtY1) ) )
mstar2<-c( crossprod(Sigma2,((L2m2)+XtY2) ) )
beta1<-mvrnorm(1,mstar1,Sigma1)
beta2<-mvrnorm(1,mstar2,Sigma2)

# Sampling of vector r

for(j in 1:n)
  {
    t.aux<-data$theta[j]
    mu.b1<-c(crossprod(beta1,X.1[j,]))
    mu.b2<-c(crossprod(beta2,X.2[j,]))
    r[j]<-Metro(llrt,t.aux,mu.b1,mu.b2,1,5)
  }

Y<-r*datose


#---- Values of each iteration ------------------------------------------------

flag<-(k/t.lag)-trunc(k/t.lag)
if(flag==0)
{
ii<-k/t.lag

B1[ii,]<-beta1
B2[ii,]<-beta2
}
#-------------  Gibbs completed ------------------------------------------------
}
#-------------------------------------------------------------------------------
# Computing the predictive measure Lk.

Lk<-0.0
flag.LK<-ifelse(flag.lk=="TRUE",1,0)

if(flag.LK){

mu<-matrix(0,tm,2)
predictiva<-rep(0,n)

for(i in 1:n){
mu[,1]<-c(crossprod(t(B1),X.1[i,]))
mu[,2]<-c(crossprod(t(B2),X.2[i,]))
norm2.mu.i<-norm2.row(mu)
vtmu.i<-c(crossprod(t(mu),datose[i,]))
predic.k<-(1/(2*pi))*exp(-0.5*norm2.mu.i)*( 1 + ((vtmu.i*pnorm(vtmu.i))/dnorm(vtmu.i)) )
predictiva[i]<-mean(predic.k)
}
Lk<-prod(predictiva)

}
#-------------------------------------------------------------------------------

# Out:

B<-list(BI=B1,BII=B2,Lk=Lk)
drop(B)

}

#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
#

Dbd<-function(t,mu1,mu2) { cos(t)*mu1+sin(t)*mu2  }

# Natural logarithm of kernel of f(ln r|theta).
llrt<-function(y,t,mu1,mu2) {
        2*y-0.5*exp(y)*(exp(y)-2*Dbd(t,mu1,mu2) )
}

## Initial values to Metropolis' algorithm.

media0<-function(t,mu1,mu2) {
     log(       (      Dbd(t,mu1,mu2)  +  (   (Dbd(t,mu1,mu2)^2)  +  8   ) ^0.5    )/2   )    }

var0<-function(m0)  {
     (   2 +  exp(2*m0)    )^(-1)  }


# Squared Euclidean norm of a matrix A, row by row.
norm2.row<-function(A){
n<-length(A[,1])
A2<-A^2
norm2<-rep(0,n)
norm2<-sapply((1:n),function(x){sum(A2[x,])})
drop(norm2)
}
#-------------------------------------------------------------------------------

## Metropolis' algorithm.

Metro<-function(f,t,mu1,mu2,tamuestra,nodeite)
{
  N<-tamuestra
  ite<-nodeite

  m0<-media0(t,mu1,mu2)
  v0<-var0(m0)
  y0<-rnorm(N,m0,sqrt(v0))

 for (i in 1:ite)
    {

     y1<-rnorm(N,m0,sqrt(v0))
     lfy1 <- f(y1,t,mu1,mu2)
     ldny1 <- log(dnorm(y1,m0,sqrt(v0)))
     w1<-(lfy1-ldny1)
     lfy0 <- f(y0,t,mu1,mu2)
     ldny0 <- log(dnorm(y0,m0,sqrt(v0)))
     w0<-(lfy0 - ldny0)
     lalpha<-(w1-w0)
     u<-runif(N,0,1)
     aux<-ifelse(log(u)<=lalpha,y1,y0)
     y0<-aux

    }

  rdt<-exp(y0)
  drop(rdt)

}
#-------------------------------------------------------------------------------

rNPxy.1<-function(M,V){
# This function simulates one observation (r,theta)
# from a 2-dimensional projected normal distribution,
# with mean vector M and covariance matrix V.

library(MASS)
xy<-mvrnorm(1,M,V)
rr<-sqrt(sum(xy^2))
ttheta<-(atan2(xy[2],xy[1]))%%(2*pi)
xy.rt<-c(rr,ttheta)
drop(xy.rt)
}

#-------------------------------------------------------------------------------

circ.stats<-function(data,flag) {
#This function calculates the sample mean direction (in degrees) of a sample of angles.
# data: Sample of angles.
# flag: If data are in degrees, set flag=0.
#       If data are in radians, set flag=1.

if (flag==0) data<-data*(pi/180)
Cbarra<-mean(cos(data))
Sbarra<-mean(sin(data))
Rbarra<-sqrt(Cbarra^2 + Sbarra^2)
auxmean<-atan(Sbarra/Cbarra)
meanphi<-0*c(1:length(data))
if (Cbarra<0) meanphi<-auxmean+pi else {
if (auxmean<0) meanphi<-auxmean+(2*pi) else meanphi<-auxmean }
theta.barra<-meanphi*180/pi
drop(theta.barra)
}


#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------

# References:

#  R Development Core Team (2009). R: A language and environment for
#  statistical computing. R Foundation for Statistical Computing,
#  Vienna, Austria. ISBN 3-900051-07-0, URL http://www.R-project.org.

